<?php
/**
 * redicon.pl
 * @author Patryk <patryk@redicon.pl>
 * @copyright redicon.pl
 * @license redicon.pl
 */

$autoloadPath = 'vendor/autoload.php';
if (file_exists($autoloadPath)) {
    require_once $autoloadPath;
}

use PrestaShop\Module\Rediconpaypo\Paypo;

require_once _PS_MODULE_DIR_ . "rediconpaypo/classes/PaypoTransaction.php";
require_once _PS_MODULE_DIR_ . "rediconpaypo/classes/Statustransaction.php";



class RediconPaypoValidationModuleFrontController extends ModuleFrontController
{
    public function postProcess()
    {
        $cart = $this->context->cart;
        $context = $this->context;

        if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active) {
            $this->errors[] = $this->l('Brak wybranego produktu na stanie lub sprawdź poprawność wymaganych danych (adresy,dane kupującego)');
            $this->redirectWithNotifications('index.php?controller=cart&action=show');
        }
        
        $authorized = false;
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == 'rediconpaypo') {
                $authorized = true;
                break;
            }
        }

        if (!$authorized) {
            die('This payment method is not available.');
        }

        $customer = new Customer($cart->id_customer);

        if (!Validate::isLoadedObject($customer)) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        $currency = $this->context->currency;
        $total = (float)$cart->getOrderTotal(true, Cart::BOTH);

        $mailVars = array();
        
        $this->module->validateOrder(
            (int)$cart->id,
            Configuration::get('REDICON_PAYPO_STATUS_NEW'),
            $total,
            $this->module->displayName,
            null,
            $mailVars,
            (int)$currency->id,
            false,
            $customer->secure_key
        );

        try {
            $order_id = (int) $this->module->currentOrder;
            $order = new Order($order_id);
            $paypo = new Paypo($order, $this->settings());

            $transaction = $paypo->createTransaction((int) $this->module->id);

            if (isset($transaction['response']) && $transaction['response']->getRedirectUrl()) {
                $obj = new PaypoTransaction();
                $obj->create((int) $cart->id, $order_id, $transaction);
                $redirectUrl = $transaction['response']->getRedirectUrl();
            } else {
                $this->errors[] = $this->l('Ups... nie udało się wykonać płatności z paypo.');
                $redirectUrl = $context->link->getModuleLink('rediconpaypo', 'clone', array('id_cart' => $order->id_cart), true);
            }
        } catch (Exception $e) {
            file_put_contents(_PS_MODULE_DIR_ . "rediconpaypo/errors", sprintf('[%s] - %s', date("Y-m-d"), $e->getMessage())."\n", FILE_APPEND);

            $msgs = [
                'Bad Request' => 'Bład płatności próbuj ponownie lub skontaktuj się ze sprzedawcą.'
            ];
            $msg = isset($msgs[$e->getMessage()])? $msgs[$e->getMessage()] : $e->getMessage();

            $this->errors[] = $msg;
            $redirectUrl = $context->link->getModuleLink('rediconpaypo', 'clone', array(
                'id_cart' => $order->id_cart,
                'message' => $msg
            ), true);
        }

        $this->redirectWithNotifications($redirectUrl);
    }

    private function settings()
    {
        if (Configuration::get('REDICON_PAYPO_ENVIROMENT') === 'sandbox') {
            $settings = [
                'url' => Configuration::get('REDICON_PAYPO_SANDBOX_URL'),
                'id' => Configuration::get('REDICON_PAYPO_SANDBOX_ID'),
                'secret' => Configuration::get('REDICON_PAYPO_SANDBOX_SECRET')
            ];
        } else {
            $settings = [
                'url' => Configuration::get('REDICON_PAYPO_PRODUCTION_URL'),
                'id' => Configuration::get('REDICON_PAYPO_PRODUCTION_ID'),
                'secret' => Configuration::get('REDICON_PAYPO_PRODUCTION_SECRET')
            ];
        }
        return $settings;
    }
}
