<?php
/**
 * redicon.pl
 * @author Patryk <patryk@redicon.pl>
 * @copyright redicon.pl
 * @license redicon.pl
 */

require_once __DIR__.'/../../../config/config.inc.php';

use PrestaShop\Module\Rediconpaypo\Paypo;

require_once _PS_MODULE_DIR_ . "rediconpaypo/rediconpaypo.php";
require_once _PS_MODULE_DIR_ . "rediconpaypo/classes/PaypoTransaction.php";
require_once _PS_MODULE_DIR_ . "rediconpaypo/classes/Statustransaction.php";




class Notification
{
    private $postData = [];
    private $paypo = null;
    private $path = '';

    public function __construct()
    {
        $this->postData = file_get_contents('php://input');
        // $this->postData = '{"merchantId":"eab975bb-43eb-48a0-b08e-f889c012d59e","referenceId":"YCNYFJFIS","transactionId":"2d7e9989-e766-4d96-b823-a8d2a959db90","transactionStatus":"ACCEPTED","amount":3446,"lastUpdate":"2021-03-25T08:45:15+01:00","message":"Payment request awaiting confirmation"}';
        $this->path = $this->dir();
        $path = $this->path.'post';
        file_put_contents($path, $this->postData."\n", FILE_APPEND);
    }

    public function dir($custom_dir = 'logs')
    {
        $dir = date('Y'.DIRECTORY_SEPARATOR.'m'.DIRECTORY_SEPARATOR.'d');
       
        $path = dirname(__DIR__).DIRECTORY_SEPARATOR.$custom_dir.DIRECTORY_SEPARATOR.$dir;
        
        if (!is_dir($path)) {
            mkdir($path, 0777, true);
        }
        return $path.DIRECTORY_SEPARATOR;
    }

    public function checkNofity()
    {
        try {
            //https://github.com/PrestaShop/PrestaShop/issues/15503 
            global $kernel;
            if (!$kernel) {
                require_once _PS_ROOT_DIR_.'/app/AppKernel.php';
                $kernel = new \AppKernel('prod', false);
                $kernel->boot();
            }
            //koniec
            
            $transactionValueObject = Paypo::jsonDecode($this->postData);
        
            if ($transactionValueObject && $transaction = PaypoTransaction::getTransactionByTransactionid($transactionValueObject->getTransactionId())) {
                $status = new Statustransaction();

                $status->create((int)$transaction['id_transaction'], $transactionValueObject, $this->postData);
               
                if ($orderId = Order::getOrderByCartId((int)$transaction['id_cart'])) {
                    $responseStatus = $transactionValueObject->getTransactionStatus();
                  
                    if ($responseStatus === 'CANCELED') {
                        $responseStatus = 'CANCELLED';
                    }
                    if ($statusOrder = Configuration::get('REDICON_PAYPO_STATUS_'.$responseStatus)) {
                    
                        //Automatyczne potwierdzenie transakcji bez zmiany statusu
                        if (Configuration::get('REDICON_PAYPO_AUTO_COMPLETED') == '1' && $responseStatus == RediconPaypo::COMPLETED) {
                            header("HTTP/1.1 200 OK");
                            exit;
                        }
                        
                        if ($orderStatus = new Order($orderId)) {
                            //Aktualne statusy przy ktorych można zmienić status zamówienia
                            $accepted_statuses = [
                                Configuration::get('REDICON_PAYPO_STATUS_NEW'),
                                Configuration::get('REDICON_PAYPO_STATUS_PENDING'),
                                // Configuration::get('REDICON_PAYPO_STATUS_ACCEPTED')//?
                            ];

                            if (in_array($orderStatus->current_state, $accepted_statuses) && $orderStatus->current_state != $statusOrder) {
                                $orderStatus->setCurrentState($statusOrder);
                            }
                        
                            if (Configuration::get('REDICON_PAYPO_AUTO_COMPLETED') == '1') {
                                if ($orderStatus->current_state == Configuration::get('REDICON_PAYPO_STATUS_ACCEPTED') && $responseStatus == RediconPaypo::ACCEPTED) {
                                    RediconPaypo::transactionConfirm($orderStatus, $transaction['transactionId']);
                                }
                            }
                        }
                    } else {
                        header("HTTP/1.1 400 Bad request");
                        file_put_contents($this->path.'errors', "[STATUS NOT IN LIST] - '{$this->postData}'\n", FILE_APPEND);
                        exit;
                    }
                }
                header("HTTP/1.1 200 OK");
                exit;
            }
        } catch (Exception $e) {
            file_put_contents($this->path.'errors', "[" . date('Y-m-d H:i:s') . " Błąd] - '{$this->postData}'\n".$e->getMessage()."\n", FILE_APPEND);
            header("HTTP/1.1 400 Bad request");
            exit;
        }
    }
}

(new Notification())->checkNofity();
