<?php
/**
 * redicon.pl
 * @author Patryk <patryk@redicon.pl>
 * @copyright redicon.pl
 * @license redicon.pl
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once(dirname(__FILE__) . '/vendor/autoload.php');

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;
use PrestaShop\Module\Rediconpaypo\Paypo;
use PrestaShop\Module\Rediconpaypo\Helper\ApiHelper;

require_once _PS_MODULE_DIR_ . "rediconpaypo/classes/PaypoTransaction.php";
require_once _PS_MODULE_DIR_ . "rediconpaypo/classes/Statustransaction.php";
require_once _PS_MODULE_DIR_ . "rediconpaypo/classes/ReturnTransaction.php";

class RediconPaypo extends PaymentModule
{
    public static $MODULE_NAME = 'rediconpaypo';
    public static $KEY_ACCESS = '68d154934f253g8760h199jjkhgi00138';
    /* @var boolean error */
    protected $_errors = false;
    
    public $settings = [
        'url' => null,
        'id' => null,
        'secret' => null
    ];
    const NEW = 'NEW';

    const PENDING = 'PENDING';

    const CANCELED = 'CANCELED';

    const CANCELLED = 'CANCELLED';

    const REJECTED = 'REJECTED';

    const ACCEPTED = 'ACCEPTED';

    const COMPLETED = 'COMPLETED';

    const PAID = 'PAID';

    const CONFIRMED = 'CONFIRMED';

    const TRANSACTION_STATUSES = [
        self::NEW,
        self::PENDING,
        self::CANCELLED,
        self::REJECTED,
        self::ACCEPTED,
        self::COMPLETED,
    ];

    public $STATUSES = [];

    public function __construct()
    {
        $this->name = 'rediconpaypo';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.8';
        $this->ps_versions_compliancy = array('min' => '1.7.1.0', 'max' => _PS_VERSION_);
        $this->author = 'Patryk Pawlicki - redicon.pl';
        $this->controllers = array('payment', 'validation', 'clone');
        $this->is_eu_compatible = 1;

        $this->currencies = false;
        $this->currencies_mode = 'radio';
        $this->need_instance = 1;
        $this->bootstrap = true;
        
        parent::__construct();

        $this->displayName = $this->l('Płatność Paypo');
        $this->description = $this->l('Kupujesz teraz, płacisz za 30 dni z PayPo - bez dodatkowych kosztów.');

        if (Configuration::get('REDICON_PAYPO_ENVIROMENT') === 'sandbox') {
            $this->settings = [
                'url'=>Configuration::get('REDICON_PAYPO_SANDBOX_URL'),
                'id'=>Configuration::get('REDICON_PAYPO_SANDBOX_ID'),
                'secret'=>Configuration::get('REDICON_PAYPO_SANDBOX_SECRET')
            ];
        } else {
            $this->settings = [
                'url'=>Configuration::get('REDICON_PAYPO_PRODUCTION_URL'),
                'id'=>Configuration::get('REDICON_PAYPO_PRODUCTION_ID'),
                'secret'=>Configuration::get('REDICON_PAYPO_PRODUCTION_SECRET')
            ];
        }
        $this->STATUSES = [
            self::NEW => $this->l('Paypo - nowy'),
            self::PENDING => $this->l('Paypo - procesowany'),
            self::CANCELLED => $this->l('Paypo - anulowany'),
            self::REJECTED => $this->l('Paypo - odrzucony'),
            self::ACCEPTED => $this->l('Paypo - zaakceptowany'),
            self::COMPLETED => $this->l('Paypo - zakończony'),
        ];
    }
    
    public static function errorMsg($text) {
        return sprintf('[%s] - %s',date("Y-m-d"),$text)."\n";
    }

    public static function settings()
    {
        $obj = new RediconPaypo();
        return $obj->settings;
    }

    private function getConfirmedUrl($query)
    {
        $ssl = 'http' . (isset($_SERVER['HTTPS']) ? 's' : '') . '://';
        $url = $ssl . $this->context->shop->domain . '/modules/rediconpaypo/ajax.php'.$query;
        return $url;
    }

    public static function transactionConfirm(Order $order, string $transactionId)
    {
        try {
            $response = (new Paypo($order, self::settings()))->updateTransaction($transactionId, self::COMPLETED);

            return $response;
        } catch (Exception $e) {
            file_put_contents(_PS_MODULE_DIR_ . "rediconpaypo/errors",  self::errorMsg($e->getMessage()), FILE_APPEND);
        }
        return false;
    }

    public static function transactionRefound(Order $order, string $transactionId, int $amount)
    {
        try {
            $response = (new Paypo($order, self::settings()))->refundsTransaction($transactionId, $amount);

            return $response;
        } catch (Exception $e) {
            file_put_contents(_PS_MODULE_DIR_ . "rediconpaypo/errors", self::errorMsg($e->getMessage()), FILE_APPEND);
        }
        return false;
    }

    public function hookPaymentReturn($params)
    {
        $order = $params['order'];

        if ($order->id) {
            $carrier = new Carrier($order->id_carrier);
        
            $replace = [
                '[ORDER_ID]' => $order->id,
                '[REFERENCE]' => $order->reference,
                '[TOTALS]' => Tools::ps_round($order->getOrdersTotalPaid(), 2),
                '[PAYMENT]' => $order->payment,
                '[SHIPPING]' => $carrier->name
            ];

            $text = Configuration::get('REDICON_PAYPO_CONFIRMATION');
            $text = str_replace(array_keys($replace), array_values($replace), $text);

            return $text;
        }
        return '';
    }

    public function hookDisplayAdminOrderMainBottom($params)
    {
        return $this->hookDisplayAdminOrder($params);
    }

    public function hookDisplayAdminOrder($params)
    {
        if (isset($params['id_order']) && is_null($params['cart']->id)) {
            $order = new Order($params['id_order']);
            $params['cart'] = new Cart($order->id_cart);
        }
        
        $cart_id = (int)$params['cart']->id;
        $order = Order::getByCartId($cart_id);

        if ($order->id && $order->module === $this->name) {
            $transaction = PaypoTransaction::getByCartId($cart_id);

            $transactionId = isset($transaction['transactionId']) ? $transaction['transactionId'] : null;
            $id_transaction = isset($transaction['id_transaction']) ? $transaction['id_transaction'] : null;

            $notifications = Statustransaction::getByTransactionId($transactionId);
            $max = ((int) PaypoTransaction::getTotalByOrderId($order->id)) / 100;
            $id_employee = $this->context->employee->id;

            $this->context->smarty->assign([
                'title_payment' => $this->l('PayPo - transakcje'),
                'title_return' => $this->l('Zwrócone kwoty:'),
                'text_return' => $this->l('Zwrot paypo'),
                'text_save' => $this->l('Zapisz'),
                'text_return_modal' => $this->l('Podaj kwotę zwrotu'),
                'text_return_label' => $this->l('Kwota zwtoru - max: ') . $max,
                'text_before' => $this->l('kwota przed: '),
                'text_after' => $this->l('zwrócono: '),
                'returns' => ReturnTransaction::getReturns($id_transaction),
                'transaction' => $transaction,
                'notifications' => $notifications,
                'id_order' => $order->id,
                'id_transaction' => $id_transaction,
                'max_value' => number_format($max, 2, '.', ''),
                'show_returns' => Configuration::get('REDICON_PAYPO_RETURNS') == '0',
                'return_ajax_url' => $this->getConfirmedUrl('?token='.self::$KEY_ACCESS.'&id_employee=' . $id_employee . '&id_order='.$order->id.'&id_transaction='.$id_transaction)
            ]);
         
            return $this->display(__FILE__.'', 'views/templates/admin/order.tpl');
        }
       
        return '';
    }

    public function hookActionOrderStatusUpdate($params)
    {
        $order = new Order($params['id_order']);

        if ($order->module == $this->name) {
            /**
             * Statusy do anulowania transakcji
             */
            $statuses = [
                (int) Configuration::get('REDICON_PAYPO_STATUS_CANCELLED'),
                (int) Configuration::get('REDICON_PAYPO_STATUS_REJECTED'),
                (int) Configuration::get('PS_OS_CANCELED')
            ];
            /**
             * Dopuszczalne statusy
             */
            $statusAccepted = [(int) Configuration::get('REDICON_PAYPO_STATUS_COMPLETED')];
            $allStatuses = array_merge($statuses, $statusAccepted);

            $newStatus = (int) $params['newOrderStatus']->id;
            $cartId = (int) $order->id_cart;
            /**
             * Jeśli aktualny status jest w liście
             *
             */
            if (in_array($newStatus, $allStatuses)) {
                $transactionId = PaypoTransaction::getTransactionIdByCartId($cartId);
            
                $paypoStatus = false;

                /**
                 * Potwierdzenie statusu w paypo po stworzeniu zamówienia
                 */
                if (Configuration::get('REDICON_PAYPO_AUTO_COMPLETED') == '0' && in_array($newStatus, $statusAccepted)) {
                    /**
                     * Akceptacja transakcji dopiero kiedy ma status accepted
                     */
                    $transactionIsAccepted = Statustransaction::getByTransactionIdAndStatus($transactionId, self::ACCEPTED);
                    if ($transactionIsAccepted) {
                        $paypoStatus = self::COMPLETED;
                    }
                }

                /**
                 * anulowanie zamówienia w paypo po anulowaniu w prescie
                 */
                if (in_array($newStatus, $statuses)) {
                    $paypoStatus = self::CANCELED;
                }

                if ($paypoStatus) {
                    try {
                        (new Paypo($order, RediconPaypo::settings()))
                        ->updateTransaction($transactionId, $paypoStatus);
                    } catch (Exception $e) {
                        file_put_contents(_PS_MODULE_DIR_ . "rediconpaypo/errors", self::errorMsg($e->getMessage()), FILE_APPEND);
                    }
                }
            }
        }
        return false;
    }

    public function hookActionValidateCustomerAddressForm($params)
    {
        $form = $params['form'];
        $phoneField = $form->getField("phone");
        $phoneMobileField = $form->getField("phone_mobile");
        $message = $this->l('Poprawny format nr telefonu to 000000000');

        if (is_null($phoneField) && is_null($phoneMobileField)) {
            $message = $this->l('Pole nr telefonu jest wymagane');
            $this->context->controller->errors[] = $message;
            return "0";
        }

        $phone =  $phoneField ? $phoneField->getValue() : '';

        if (empty($phone) && !is_null($phoneMobileField)) {
            $phoneField = $form->getField("phone_mobile");
            $phone = $phoneField ? $phoneField->getValue() : '';
        }

        if ($phone && !ApiHelper::formatPhone($phone)) {
            if ($phoneField) {
                $phoneField->addError($message);
            }

            $this->context->controller->errors[] = $message;
            return "0";
        }

        return "1";
    }

    public function hookPaymentOptions($params)
    {
        $curency = new Currency($params['cart']->id_currency);

        if (!in_array($curency->iso_code, ["PLN"])) {
            return false;
        }

        $errors = Tools::getValue('message', '');
        $message = '';
        if ($errors) {
            $message = sprintf('<strong class="text-danger">%s</strong>', $errors);
        }

        $paymentOptions = [];
        $paymentOption = new PaymentOption();
        $paymentOptions[] = $paymentOption
            ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/views/img/logo.png'))
            ->setCallToActionText($this->l('Zapłać z PayPo | Kup teraz, zapłać za 30 dni'))
            ->setModuleName($this->name)
            ->setAction($this->context->link->getModuleLink($this->name, 'validation', array(), true))
            ->setAdditionalInformation($message . Configuration::get('REDICON_PAYPO_DESCRIPTION', ''));
           
        return $paymentOptions;
    }

    public function addReturns($params)
    {
        $order = new Order($params['id_order']);

        if ($order->id) {
            $id_order = (int)$order->id;

            $current_total = (int) PaypoTransaction::getTotalByOrderId($id_order);
            $transaction = PaypoTransaction::getByOrderId($id_order);
            $return_amount = round($params['amount']*100);

            if ($current_total >= $return_amount) {
                $save_total = $current_total - $return_amount;
                @file_put_contents(_PS_MODULE_DIR_ . "rediconpaypo/total", "return_amount: $return_amount, before:$save_total,save_total:$save_total     \n", FILE_APPEND);

                if (isset($transaction['transactionId']) && isset($transaction['id_transaction'])) {
                    $response = self::transactionRefound($order, $transaction['transactionId'], $return_amount);

                    if ($response && $response->getCode()) {
                        $id_employee = $params['id_employee'];

                        if ($status = Configuration::get('REDICON_PAYPO_STATUS_STATUS_RETURN')) {
                            $order->setCurrentState($status);
                        }

                        $save_return = new ReturnTransaction();
                        $save_return->create((int) $transaction['id_transaction'], $current_total, $return_amount, $id_employee);
                        
                        /**
                         * Aktualizowanie nowej wartości koszyka jeśli zwrot został przyjęty
                         */
                        $trans = new PaypoTransaction($transaction['id_transaction']);
                        $trans->total = $save_total;
                        $trans->transactionId = $transaction['transactionId'];
                        $trans->id_order = $id_order;
                        $trans->updated_at = date('Y-m-d H:i:s');
                        $trans->update();
                        
                        return 'success';
                    }
                }
            }

            return ($current_total ? $current_total / 100 : 0);
        }

        return false;
    }

    public function hookActionObjectUpdateAfter($params)
    {
        if ($params['object'] instanceof Order) {
            $order = $params['object'];
            if (Configuration::get('REDICON_PAYPO_RETURNS')) {
                if ($order->id && $order->module === $this->name) {
                    $id_order = (int)$order->id;

                    $save_total = (int) PaypoTransaction::getTotalByOrderId($id_order);
                    $transaction = PaypoTransaction::getByOrderId($id_order);
                    $current_total = round($order->getOrdersTotalPaid()*100);
                
                    /**
                     * Jeśli się zmieni kwota zamówienia na mniejszą wysłać info do paypo
                     */
                    if ($current_total < $save_total) {
                        $spread = (int) ($save_total - $current_total);

                        file_put_contents(_PS_MODULE_DIR_ . "rediconpaypo/total", "spread: $spread,current_total:$current_total,save_total:$save_total     \n", FILE_APPEND);
                        
                        if ($spread > 0) {
                            if (isset($transaction['transactionId'])) {
                                $response = self::transactionRefound($order, $transaction['transactionId'], $spread);
                                if ($response && $response->getCode()) {
                                    /**
                                     * Aktualizowanie nowej wartości koszyka jeśli zwrot został przyjęty
                                     */
                                    $trans = new PaypoTransaction($transaction['id_transaction']);
                                    $trans->total = $current_total;
                                    $trans->transactionId = $transaction['transactionId'];
                                    $trans->id_order = $id_order;
                                    $trans->updated_at = date('Y-m-d H:i:s');
                                    $trans->update();
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    public function getContent()
    {
        $this->savePost();
        return  $this->renderForm();
    }

    private function script(){
        return '$(document).ready(function(){if($(\'[name="REDICON_PAYPO_PRODUCT_TYPE"]\').val()==\'CORE\'){
                $(\'[name="REDICON_PAYPO_INSTALLMENT_COUNT"]\').closest(\'.form-group\').hide()
                $(\'[name="REDICON_PAYPO_INSTALLMENT_COUNT"]\').val(4)
            }
        $(\'[name="REDICON_PAYPO_PRODUCT_TYPE"]\').change(function(){
            if($(this).val()==\'CORE\'){
                $(\'[name="REDICON_PAYPO_INSTALLMENT_COUNT"]\').closest(\'.form-group\').hide()
                $(\'[name="REDICON_PAYPO_INSTALLMENT_COUNT"]\').val(4)
            }else{
                $(\'[name="REDICON_PAYPO_INSTALLMENT_COUNT"]\').closest(\'.form-group\').show()
                $(\'[name="REDICON_PAYPO_INSTALLMENT_COUNT"]\').val(1)
            }
        });});';
    }

    protected function renderForm()
    {
        $default_lang = (int) Configuration::get('PS_LANG_DEFAULT');
        $desc = 'Wymagane ustawienie statusów według schematu';

        if ($this->STATUSES) {
            foreach ($this->STATUSES as $key => $s) {
                $desc .= "<br/>Status-$key - $s";
            }
        }

        $tags  = '<br/> [ORDER_ID] - id zamówienia';
        $tags .= '<br/> [TOTALS] - kwota do zapłaty';
        $tags .= '<br/> [REFERENCE] - ID referencyjny';
        $tags .= '<br/> [PAYMENT] - metoda płatności';
        $tags .= '<br/> [SHIPPING] - metoda wysyłki';

        $fields_form = array(
            'tinymce' => true,
            'legend' => array(
                'title' => $this->l('Dynamiczne scrypty - header'),
            ),
            'input' => array(
                'description' => array(
                    'type' => 'textarea',
                    'label' => $this->l('Opis / instrukcja płatności'),
                    'name' => 'REDICON_PAYPO_DESCRIPTION',
                    'cols' => 40,
                    'rows' => 10,
                    'autoload_rte' => 'rte'
                ),
                'confirmation' => array(
                    'type' => 'textarea',
                    'label' => $this->l('Podsumowaniem zamówienia'),
                    'name' => 'REDICON_PAYPO_CONFIRMATION',
                    'cols' => 40,
                    'rows' => 10,
                    'autoload_rte' => 'rte',
                    'desc' => $this->l('Użyj tagi aby dynamicznie podmienić dane w tekście:') . $tags
                ),
                'enviroment' => array(
                    'type' => 'select',
                    'lang' => true,
                    'label' => $this->l('Tryb sandbox / produkcja'),
                    'name' => 'REDICON_PAYPO_ENVIROMENT',
                    'desc' => $this->l('sandbox / produkcja'),
                    'options' => array(
                        'query' => [
                            [
                                'id_option' => 'sandbox',
                                'name' => 'Sandbox'
                            ],
                            [
                                'id_option' => 'production',
                                'name' => 'Produkcja'
                            ]
                        ],
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                'AUTO_COMPLETED' => [
                    'type' => 'switch',
                    'lang' => true,
                    'label' => $this->l('Automatyczne zatwierdzanie transakcji w paypo'),
                    'hint' => $this->l('Wymagane jest ustawienie wszystkich statusów zgodnie z instrukcją.'),
                    'name' => 'REDICON_PAYPO_AUTO_COMPLETED',
                    'desc' => $desc,
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'REDICON_PAYPO_AUTO_COMPLETED_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'REDICON_PAYPO_AUTO_COMPLETED_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    ),
                    ],
                'REDICON_PAYPO_RETURNS' => array(
                    'type' => 'switch',
                    'label' => $this->l('Automatyczne zwroty produktów'),
                    'name' => 'REDICON_PAYPO_RETURNS',
                    'required' => false,
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'REDICON_PAYPO_RETURNS_on',
                            'value' => 1,
                            'label' => $this->l('Enabled'),
                        ),
                        array(
                            'id' => 'REDICON_PAYPO_RETURNS_off',
                            'value' => 0,
                            'label' => $this->l('Disabled'),
                        ),
                    ),
                    
                    'desc' => $this->l('Przy ustawionej opcji TAK zwrot/edycja ilości produktu będzie przekazana do paypo')
                ),
                'REDICON_PAYPO_PRODUCT_TYPE' => array(
                    'type' => 'select',
                    'lang' => true,
                    'label' => $this->l('Typ produktu'),
                    'name' => 'REDICON_PAYPO_PRODUCT_TYPE',
                    'desc' => '<script>'.$this->script().'</script>',
                    'options' => array(
                        'query' => [
                            [
                            "id_option" => "CORE",
                            "name" => "CORE"
                            ],
                            [
                            "id_option" => "PNX",
                            "name" => "PNX"
                            ]
                    ],
                        'id' => 'id_option',
                        'name' => 'name'
                    )
                ),
                'REDICON_PAYPO_INSTALLMENT_COUNT' => array(
                    'type' => 'text',
                    'label' => $this->l('Ilość rat'),
                    'name' => 'REDICON_PAYPO_INSTALLMENT_COUNT',
                    'class' => 'form-control',
                    'desc' => $this->l('liczba większa od 1!')
                ),
                //PS_ORDER_RETURN
                'label_production'=>array(
                    'type' => 'free',
                    'label' => '',
                    'name' => 'PRODUCTION',
                ),
                'production_id' => array(
                    'type' => 'text',
                    'label' => $this->l('Identyfikator (produkcja)'),
                    'name' => 'REDICON_PAYPO_PRODUCTION_ID',
                    'class' => 'form-control',
                    'desc' => $this->l('xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx')
                ),
                'production_secret' => array(
                    'type' => 'text',
                    'label' => $this->l('Secret (producja)'),
                    'name' => 'REDICON_PAYPO_PRODUCTION_SECRET',
                    'class' => 'form-control',
                    'desc' => $this->l('xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx')
                ),
                'production_url' => array(
                    'type' => 'text',
                    'label' => $this->l('url api (produkcja)'),
                    'name' => 'REDICON_PAYPO_PRODUCTION_URL',
                    'class' => 'form-control',
                    'desc' => $this->l('https://api.paypo.pl/v3/')
                ),
                'label_sandbox'=>array(
                    'type' => 'free',
                    'label' => '',
                    'name' => 'SANDBOX',
                ),
                'sandbox_id' => array(
                    'type' => 'text',
                    'label' => $this->l('Identyfikator (sandbox)'),
                    'name' => 'REDICON_PAYPO_SANDBOX_ID',
                    'class' => 'form-control',
                    'desc' => $this->l('xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx')
                ),
                'sandbox_secret' => array(
                    'type' => 'text',
                    'label' => $this->l('Secret (sandbox)'),
                    'name' => 'REDICON_PAYPO_SANDBOX_SECRET',
                    'class' => 'form-control',
                    'desc' => $this->l('xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx')
                ),
                'sandbox_url' => array(
                    'type' => 'text',
                    'label' => $this->l('url api (sandbox)'),
                    'name' => 'REDICON_PAYPO_SANDBOX_URL',
                    'class' => 'form-control',
                    'desc' => $this->l('https://api.sandbox.paypo.pl/v3/')
                ),
                'label_url'=>array(
                    'type' => 'free',
                    'label' => '',
                    'name' => 'STATUS',
                ),
            ),
            'submit' => array(
                'title' => $this->l('zapisz'),
            )

        );

        $statuses = $this->getStatuses();

        $fields_form['input']['STATUS_RETURN']=[
            'type' => 'select',
            'lang' => true,
            'label' => $this->l('Status-RETURN'),
            'name' => 'REDICON_PAYPO_STATUS_STATUS_RETURN',
            'desc' => $this->l('Status po ręcznym zwrocie transakcji do paypo'),
            'options' => array(
                'query' => array_merge([
                    [
                    "id_option" => "0",
                    "name" => "Brak"
                ]
            ], $statuses),
                'id' => 'id_option',
                'name' => 'name'
            )
        ];

        foreach (self::TRANSACTION_STATUSES as $s) {
            $desc = ($s === self::COMPLETED?$this->l('Status przy którym wysyłamy potwierdzenie transakcji do paypo'):'');
            $desc = ($s === self::CANCELLED?$this->l('Ustawienie tego statusu w zamówieniu spowoduje anulowanie transakcji w paypo'):$desc);

            $fields_form['input']['STATUS_'.$s]=[
                    'type' => 'select',
                    'lang' => true,
                    'label' => $this->l('Status-'.$s),
                    'name' => 'REDICON_PAYPO_STATUS_'.$s,
                    'desc' => $desc,
                    'options' => array(
                        'query' => $statuses,
                        'id' => 'id_option',
                        'name' => 'name'
                    )
            ];
        }

        if (Shop::isFeatureActive() && Tools::getValue('id_info') == false) {
            $fields_form['input'][] = array(
                'type' => 'shop',
                'label' => $this->l('Shop association'),
                'name' => 'checkBoxShopAsso_theme'
            );
        }


        $helper = new HelperForm();
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->identifier = $this->identifier;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;
        $helper->toolbar_scroll = true;
        $helper->title = $this->displayName;
        $helper->submit_action = 'save' . $this->name;

        $desc_default = '<p><img src="'.Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/views/img/logo_PayPo.png').'" width="50%" alt="rediconpaypo.logo.png" /></p>
        <p class="MsoNormal">Kupujesz teraz, płacisz za 30 dni z PayPo - bez dodatkowych kosztów.</p>
        <p class="MsoNormal">Po 30 dniach możesz również rozłożyć płatność na wygodne raty. </p>
        <p class="MsoNormal">Do ukończenia transakcji podajesz swoje dane w celu weryfikacji tożsamości.</p>
        <p class="MsoNormal">Wtedy PayPo opłaci Twój rachunek i odroczy Twoją płatność.</p>';
        
        $helper->fields_value = [
            'PRODUCTION' => '<br/><h3>Ustawienia produkcja</h3>',
            'SANDBOX' => '<br/><h3>Ustawienia sandbox</h3>',
            'STATUS' => '<br/><h3>Ustawienia statusów</h3>',
            'REDICON_PAYPO_PRODUCT_TYPE' => Configuration::get('REDICON_PAYPO_PRODUCT_TYPE', $default_lang, null, null, 'core'),
            'REDICON_PAYPO_INSTALLMENT_COUNT' => Configuration::get('REDICON_PAYPO_INSTALLMENT_COUNT', $default_lang, null, null, '1'),
            'REDICON_PAYPO_RETURNS' => Configuration::get('REDICON_PAYPO_RETURNS', $default_lang, null, null, '1'),
            'REDICON_PAYPO_DESCRIPTION' => Configuration::get('REDICON_PAYPO_DESCRIPTION', $default_lang, null, null, $desc_default),
            'REDICON_PAYPO_CONFIRMATION' => Configuration::get('REDICON_PAYPO_CONFIRMATION', $default_lang, null, null, ''),
            'REDICON_PAYPO_ENVIROMENT' => Configuration::get('REDICON_PAYPO_ENVIROMENT', $default_lang, null, null, ''),
            'REDICON_PAYPO_AUTO_COMPLETED' => Configuration::get('REDICON_PAYPO_AUTO_COMPLETED', $default_lang, null, null, '1'),
            'REDICON_PAYPO_PRODUCTION_ID' => Configuration::get('REDICON_PAYPO_PRODUCTION_ID', $default_lang, null, null, ''),
            'REDICON_PAYPO_PRODUCTION_SECRET' => Configuration::get('REDICON_PAYPO_PRODUCTION_SECRET', $default_lang, null, null, ''),
            'REDICON_PAYPO_PRODUCTION_URL' => Configuration::get('REDICON_PAYPO_PRODUCTION_URL', $default_lang, null, null, ''),
            'REDICON_PAYPO_SANDBOX_ID' => Configuration::get('REDICON_PAYPO_SANDBOX_ID', $default_lang, null, null, ''),
            'REDICON_PAYPO_SANDBOX_SECRET' => Configuration::get('REDICON_PAYPO_SANDBOX_SECRET', $default_lang, null, null, ''),
            'REDICON_PAYPO_SANDBOX_URL' => Configuration::get('REDICON_PAYPO_SANDBOX_URL', $default_lang, null, null, ''),
        ];

        $helper->fields_value['REDICON_PAYPO_STATUS_STATUS_RETURN'] = Configuration::get('REDICON_PAYPO_STATUS_STATUS_RETURN', $default_lang, null, null, '');
        
        foreach (self::TRANSACTION_STATUSES as $s) {
            $key = 'REDICON_PAYPO_STATUS_'.$s;
            $helper->fields_value['REDICON_PAYPO_STATUS_'.$s] = Configuration::get($key, $default_lang, null, null, '');
        }

        $url = $this->getModuleUrl('&date='.date('Y/m/d'));
        
        $errors_file = sprintf('<a href="%s">logi wtyczki:</a>', $url);

        if (Tools::getValue('date')) {
            if (file_exists(dirname(__FILE__).'/errors')) {
                $errors_file .= sprintf('<br/><a href="%s">error_logi</a>', __PS_BASE_URI__.'modules/'.$this->name.'/errors');
            }
       
            if (file_exists(dirname(__FILE__).'/logs/'.Tools::getValue('date').'/post')) {
                $file = 'modules/'.$this->name.'/logs/'.Tools::getValue('date').'/post';
                $errors_file .= sprintf('<br/><a href="%s">file_%s</a>', __PS_BASE_URI__ . $file, str_replace(['/'], ['_'], Tools::getValue('date')));
            }
        }

        return $helper->generateForm(array(array('form' => $fields_form))).$errors_file;
    }

    private function getStatuses()
    {
        return array_map(function ($t) {
            return [
                'id_option' => $t['id_order_state'],
                'name' => $t['name'],
            ];
        }, OrderState::getOrderStates(Configuration::get('PS_LANG_DEFAULT')));
    }

    private function savePost()
    {
        if (Tools::isSubmit('save' . $this->name)) {
        }
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {
            if (Tools::isSubmit('save' . $this->name)) {
                foreach ($_POST as $key => $value) {
                    if (strpos($key, 'REDICON') !== false) {
                        Configuration::updateValue($key, is_array($value)?implode(',', $value):$value, strpos($key, 'DESCRIPTION') || strpos($key, 'CONFIRMATION'));
                    }
                }

                Tools::redirectAdmin($this->getModuleUrl());
            }
        }
    }

    private function getModuleUrl($uri = '')
    {
        return 'index.php?controller=AdminModules&configure='.$this->name.'&token='.Tools::getAdminTokenLite('AdminModules').$uri;
    }

    public function install()
    {
        if (version_compare(_PS_VERSION_, '1.7.7', '>=')) {
            $admin_hook = 'displayAdminOrderMain';
        } else {
            $admin_hook = 'displayAdminOrder';
        }

        return parent::install() &&
            $this->sqlInstall() &&
            $this->installOrderState() &&
            $this->registerHook('displayPayment') &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook($admin_hook) &&
            $this->registerHook('actionOrderStatusUpdate') &&
            $this->registerHook('actionValidateCustomerAddressForm') &&
            $this->registerHook('paymentReturn') &&
            $this->registerHook('actionObjectUpdateAfter');
    }

    public function uninstall()
    {
        $sql1 = 'DROP TABLE IF EXISTS `'._DB_PREFIX_.'paypo_transactions`';
        $sql2 = 'DROP TABLE IF EXISTS `'._DB_PREFIX_.'paypo_status_transaction`';
        $sql3 = 'DROP TABLE IF EXISTS `'._DB_PREFIX_.'paypo_returns_transaction`';
        
        if (!parent::uninstall() or
        !Db::getInstance()->execute($sql1) or
        !Db::getInstance()->execute($sql2) or
        !Db::getInstance()->execute($sql3)) {
            return false;
        }

        return true;
    }

    private function sqlInstall()
    {
        $sqls = [
            "CREATE TABLE IF NOT EXISTS `"._DB_PREFIX_."paypo_transactions` (
                `id_transaction` int(11) NOT NULL AUTO_INCREMENT,
                `referenceId` varchar(40) DEFAULT NULL,
                `transactionId` varchar(36) DEFAULT NULL,
                `redirectUrl` varchar(500) DEFAULT NULL,
                `id_cart` int(11) NOT NULL DEFAULT '0',
                `id_order` int(11) NOT NULL DEFAULT '0',
                `total` int(11) NOT NULL DEFAULT '0',
                `json` text,
                `completed` tinyint(4) NOT NULL DEFAULT '0',
                `created_at` datetime DEFAULT NULL,
                `updated_at` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                PRIMARY KEY (`id_transaction`)
              ) ENGINE=InnoDB DEFAULT CHARSET=utf8;",
              "CREATE TABLE IF NOT EXISTS `"._DB_PREFIX_."paypo_status_transaction` (
                `id_status` int(11) NOT NULL AUTO_INCREMENT,
                `id_transaction` int(11) NOT NULL,
                `referenceId` varchar(40) DEFAULT NULL,
                `transactionId` varchar(36) DEFAULT NULL,
                `transactionStatus` varchar(20) NOT NULL,
                `amount` int(11) NOT NULL,
                `message` varchar(500) DEFAULT NULL,
                `json` text,
                `created_at` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                PRIMARY KEY (`id_status`)
              ) ENGINE=InnoDB DEFAULT CHARSET=utf8;",
              "CREATE TABLE IF NOT EXISTS `"._DB_PREFIX_."paypo_returns_transaction` (
                `id_return` int(11) NOT NULL AUTO_INCREMENT,
                `id_transaction` int(11) NOT NULL,
                `before_amount` int(11) NOT NULL,
                `amount` int(11) NOT NULL,
                `id_employee` int(11) NOT NULL,
                `created_at` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                PRIMARY KEY (`id_return`)
              ) ENGINE=InnoDB DEFAULT CHARSET=utf8;"
        ];

        $result = true;
        foreach ($sqls as $sql) {
            if (!Db::getInstance()->execute($sql)) {
                return false;
            }
        }
        return $result;
    }

    public function installOrderState()
    {
        foreach ($this->STATUSES as $code => $text) {
            if (!Configuration::get('REDICON_PAYPO_STATUS_'.$code) || !Validate::isLoadedObject(new OrderState(Configuration::get('REDICON_PAYPO_STATUS_'.$code)))) {
                $order_state = new OrderState();
                $order_state->name = array();
                foreach (Language::getLanguages() as $language) {
                    $order_state->name[$language['id_lang']] = $text;
                }
                $order_state->send_email = false;
                $order_state->color = '#454545';
                $order_state->hidden = false;
                $order_state->delivery = false;
                $order_state->logable = false;
                $order_state->invoice = false;
                $order_state->module_name = $this->name;

                if ($order_state->add()) {
                }

                if (Shop::isFeatureActive()) {
                    $shops = Shop::getShops();
                    foreach ($shops as $shop) {
                        Configuration::updateValue('REDICON_PAYPO_STATUS_'.$code, (int) $order_state->id, false, null, (int)$shop['id_shop']);
                    }
                } else {
                    Configuration::updateValue('REDICON_PAYPO_STATUS_'.$code, (int) $order_state->id);
                }
            }
        }

        return true;
    }
}
